__title__ = 'tello-python'
__author__ = 'C灵C'
__liscence__ = 'MIT'
__copyright__ = 'Copyright 2021 C灵C'
__version__ = '1.1.3'
__all__ = ['tello', 'stats']

from .tello import Tello
from .stats import Stats